#' Information-based generalized Robinson-Foulds distances
#'
#' Tree similarity and distance measures that measure the amount of 
#' phylogenetic or clustering information that two trees hold in common.
#' 
#' 
#' A tree similarity measure is generated by finding an optimal matching that
#' maximises the total information in common between a partition on one tree
#' and its pair on a second, considering all possible ways to pair partitions 
#' between trees (including leaving a partition unpaired).
#' 
#' The returned tree similarity measures state the amount of information, 
#' in bits, that the partitions in two trees hold in common 
#' when they are optimally matched, following Smith (forthcoming).  
#' The complementary tree distance measures state how much information is 
#' different in the partitions of two trees, under an optimal matching.
#' 
#' @section Concepts of information:
#'
#' Each partition in a tree can be viewed as a statement that the 'true' 
#' tree is one of those that splits the taxa as specified.  This concept
#' corresponds to the concept of phylogenetic information, which views the
#' information content of a pair of partitions as relating to the proportion of
#' phylogenetic trees that are consistent with both partitions.
#' 
#' This proportion gives rise to the Mutual Phylogenetic Information similarity 
#' measure (`MutualPhylogeneticInfo`), and the complementary 
#' Variation of Phylogenetic Information distance metric
#' (`VariationOfPhylogeneticInfo`).
#' 
#' An information-theoretic take on the [`MatchingSplitDistance`] offers a 
#' relaxed interpretation of this measurement, which attributes some information
#' to similar yet contradictory splits.  This metric is implemented in
#' `MutualMatchingSplitInfo` and `VariationOfMatchingSplitInfo`.
#' 
#' Alternatively, each partition can be viewed as a statement that the taxa
#' are subdivided into the two groups specified.  On this view, the 
#' information content of a pair of partitions corresponds to their
#' mutual clustering information (Meila 2007, Vinh2010), giving rise to their
#' Mutucal Clustering Information and Variation of Clustering Information
#' similarity/distance metrics (`MutualClusteringInfo`, `VariationOfClusteringInfo`).
#' 
#' This latter approach is optimal in many regards, and is implemented, normalized
#' against the total information present, in the convenience function `TreeDistance`.
#' 
#' 
#' @section Normalization:
#' 
#' If `normalize = TRUE`, then results will be rescaled from zero to a nominal
#' maximum value, calculated thus:
#' 
#' * `MutualPhylogeneticInfo`, `MutualClusteringInfo`, `MutualMatchingSplitInfo`:
#'  The information content of the least informative tree.
#'  To scale against the information content of the most informative tree, use
#' `normalize = pmax`.
#' 
#' * `VariationOfPhylogeneticInfo`, `VariationOfClusteringInfo`,
#' `VariationOfMatchingSplitInfo`: The sum of the
#' (phylogenetic or clustering) information content of the two trees.
#' 
#' @template tree12Params
#' 
#' @param normalize If a numeric value is provided, this will be used as a 
#' maximum value against which to rescale results.
#' If `TRUE`, results will be rescaled against a maximum value calculated from
#' the specified tree sizes and topology, as specified in the 'Normalization' 
#' section below.
#' If `FALSE`, results will not be rescaled.
#' 
#' @param reportMatching Logical specifying whether to return the clade
#' matchings as an attribute of the score.
#'
#' @return If `reportMatching = FALSE`, the functions return a numeric 
#' vector specifying the requested similarities or differences.
#' 
#' If `reportMatching = TRUE`, the functions additionally return details
#' of which clades are matched in the optimal matching, which can be viewed
#' using [`VisualizeMatching`].
#'  
#' @examples {
#'   tree1 <- ape::read.tree(text='((((a, b), c), d), (e, (f, (g, h))));')
#'   tree2 <- ape::read.tree(text='(((a, b), (c, d)), ((e, f), (g, h)));')
#'   tree3 <- ape::read.tree(text='((((h, b), c), d), (e, (f, (g, a))));')
#'   
#'   # Best possible score is obtained by matching a tree with itself
#'   VariationOfPhylogeneticInfo(tree1, tree1) # 0, by definition
#'   MutualPhylogeneticInfo(tree1, tree1)
#'   PartitionInfo(tree1) # Maximum mutual phylogenetic information
#'   
#'   # Best possible score is a function of tree shape; the partitions within
#'   # balanced trees are more independent and thus contain less information
#'   PartitionInfo(tree2)
#'   
#'   # How similar are two trees?
#'   MutualPhylogeneticInfo(tree1, tree2) # Amount of phylogenetic information in common
#'   VisualizeMatching(MutualPhylogeneticInfo, tree1, tree2) # Which clades are matched?
#'   VariationOfPhylogeneticInfo(tree1, tree2) # Distance measure
#'   VariationOfPhylogeneticInfo(tree2, tree1) # The metric is symmetric
#'   #'   
#'   # Are they more similar than two trees of this shape would be by chance?
#'   ExpectedVariation(tree1, tree2, sample=12)['VariationOfPhylogeneticInfo', 'Estimate']
#'   
#'   # Every partition in tree1 is contradicted by every partition in tree3
#'   # Non-arboreal matches contain clustering, but not phylogenetic, information
#'   MutualPhylogeneticInfo(tree1, tree3) # = 0
#'   MutualClusteringInfo(tree1, tree3) # > 0
#'   
#' }
#' 
#' 
#' @references {
#'  * \insertRef{Meila2007}{TreeDist}
#'  * \insertRef{SmithDist}{TreeDist}
#'  * \insertRef{Vinh2010}{TreeDist}
#' }
#' 
#' @author Martin R. Smith
#' 
#' @family tree distances
#' @export
TreeDistance <- function (tree1, tree2 = tree1) {
  MutualClusteringInfo(tree1, tree2, normalize = TRUE, reportMatching = FALSE)
}

#' @describeIn TreeDistance Mutual phylogenetic information between two trees.
#' @export
MutualPhylogeneticInfo <- function (tree1, tree2 = tree1, normalize = FALSE,
                                    reportMatching = FALSE) {
  unnormalized <- CalculateTreeDistance(MutualPhylogeneticInfoSplits, tree1,
                                        tree2, reportMatching=reportMatching)
  
  # Return:
  NormalizeInfo(unnormalized, tree1, tree2, how = normalize,
                InfoInTree = PartitionInfo, Combine = pmin)
}

#' @describeIn TreeDistance Variation of phylogenetic information between two trees.
#' @export
VariationOfPhylogeneticInfo <- function (tree1, tree2 = tree1, 
                                         normalize = FALSE,
                                         reportMatching = FALSE) {
  mai <- MutualPhylogeneticInfo(tree1, tree2, normalize = FALSE,
                                reportMatching = reportMatching)
  treesIndependentInfo <- outer(PartitionInfo(tree1), PartitionInfo(tree2), '+')
  
  ret <- treesIndependentInfo - mai - mai
  ret <- NormalizeInfo(ret, tree1, tree2, how=normalize, 
                       infoInBoth = treesIndependentInfo,
                       InfoInTree = PartitionInfo, Combine = '+')
  
  ret[ret < 1e-13] <- 0 # In case of floating point inaccuracy
  attributes(ret) <- attributes(mai)
  
  # Return:
  ret
}

#' @describeIn TreeDistance Variation of clustering information between two trees.
#' @export
VariationOfClusteringInfo <- function (tree1, tree2 = tree1, normalize = FALSE,
                                       reportMatching = FALSE) {
  mci <- MutualClusteringInfo(tree1, tree2, normalize = FALSE, 
                              reportMatching = reportMatching)
  treesIndependentInfo <- outer(ClusteringInfo(tree1), ClusteringInfo(tree2), '+')
  
  ret <- treesIndependentInfo - mci - mci
  ret <- NormalizeInfo(ret, tree1, tree2, how = normalize,
                       infoInBoth = treesIndependentInfo,
                       InfoInTree = ClusteringInfo, Combine = '+')
  
  ret[ret < 1e-13] <- 0 # In case of floating point inaccuracy
  attributes(ret) <- attributes(mci)
  
  # Return:
  ret
}



#' @describeIn TreeDistance Estimate expected Variation of 
#' Phylogenetic Information and Mutual Phylogenetic Information for a pair of trees of
#' a given topology.
#' @param samples Integer specifying how many samplings to obtain; 
#' accuracy of estimate increases with `sqrt(samples)`.
#' @importFrom stats sd
#' @importFrom TreeTools as.Splits .DecodeBinary
#' @export
ExpectedVariation <- function (tree1, tree2, samples = 1e+3) {
  info1 <- PartitionInfo(tree1)
  info2 <- PartitionInfo(tree2)
  splits1 <- as.Splits(tree1)
  splits2 <- as.Splits(tree2)
  nTip <- attr(splits2, 'nTip')
  tipLabels <- attr(splits2, 'tip.label')
  
  logical2 <- t(matrix(unlist(apply(splits2, 1, .DecodeBinary, nTip = nTip)), 
                     ncol = length(splits2), nrow = nTip))
  
  mutualEstimates <- vapply(seq_len(samples), function (x) {
    splits2 <- as.Splits(logical2[, sample.int(nTip, nTip)])
    
    c(MutualPhylogeneticInfoSplits(splits1, splits2),
      MutualMatchingSplitInfoSplits(splits1, splits2))
  }, c(MutualPhylogeneticInfo = 0, MutualMatchingSplitInfo = 0))
  
  mut <- cbind(Estimate = rowMeans(mutualEstimates),
               sd = apply(mutualEstimates, 1, sd), n = samples)
  
  ret <- rbind(
    mut,
    VariationOfPhylogeneticInfo = c(info1 + info2 - mut[1, 1] - mut[1, 1],
                                    mut[1, 2] * 2, samples),
    VariationOfMatchingSplitInfo = c(info1 + info2 - mut[2, 1] - mut[2, 1],
                                     mut[2, 2] * 2, samples)
  )
  
  # Return:
  cbind(Estimate = ret[, 1],
        'Std. Err.' = ret[, 'sd'] / sqrt(ret[, 'n']), 
        ret[, 2:3])
}

#' @describeIn TreeDistance Mutual Clustering Information of two trees.
#' @export
MutualClusteringInfo <- function (tree1, tree2, normalize = FALSE,
                                  reportMatching = FALSE) {
  unnormalized <- CalculateTreeDistance(MutualClusteringInfoSplits, tree1, tree2,
                                        reportMatching)
  NormalizeInfo(unnormalized, tree1, tree2, ClusteringInfo, 
                how = normalize, Combine = pmin)
}

#' @describeIn TreeDistance Calculate mutual phylogenetic information from splits instead of trees.
#' @template splits12params
#' @param nTip Integer specifying the number of tips in each split.
#' @export
MutualPhylogeneticInfoSplits <- function (splits1, splits2,
                                          nTip = attr(splits1, 'nTip'),
                                          reportMatching = FALSE) {
  GeneralizedRF(splits1, splits2, nTip, cpp_mutual_phylo,
                maximize = TRUE, reportMatching = reportMatching)
}

#' @describeIn TreeDistance Calculate clustering information from splits instead of trees
#' @export
MutualClusteringInfoSplits <- function (splits1, splits2,
                                        nTip = attr(splits1, 'nTip'),
                                        reportMatching = FALSE) {
  GeneralizedRF(splits1, splits2, nTip, cpp_mutual_clustering,
                maximize = TRUE, reportMatching = reportMatching)
}
